﻿using System.Security.Cryptography;
using System.Text;

namespace anbs_cp.Classes;

/// <summary>
/// Класс для работы с хэшем файла
/// </summary>
public sealed class FileHash
{
    /// <summary>
    /// Получение md5-хэша файла.
    /// Взято с https://stackoverflow.com/a/24031467/16469671
    /// </summary>
    /// <param name="fileName">Имя файла</param>
    /// <returns>Массив хэша</returns>
    public FileHash (string fileName)
    {
        using MD5 md5 = MD5.Create();
        using FileStream stream = File.OpenRead(fileName);
        Hash = md5.ComputeHash(stream);
    }

    /// <summary>
    /// Простой конструктор
    /// </summary>
    public FileHash () => Hash = [];

    /// <summary>
    /// Хэш файла
    /// </summary>
    public byte[] Hash { get; private set; }

    /// <summary>
    /// Вывод в строку
    /// </summary>
    /// <returns>Строка хэша файла</returns>
    public override string ToString () => BitConverter.ToString(Hash).Replace("-", "").ToLowerInvariant();

    /// <summary>
    /// Конвертирует строку в хэш
    /// </summary>
    /// <param name="value">Строка</param>
    public void FromString (string value)
    {
        UTF8Encoding utf8 = new();
        Hash = utf8.GetBytes(value);
    }

    /// <summary>
    /// Сравнивает текущий хэш с хэшем <paramref name="otherHash"/> и выдаёт <see cref="bool.TrueString"/>, если совпадают,
    /// и <see cref="bool.FalseString"/>, если не совпадают.
    /// </summary>
    /// <param name="otherHash">Другой хэш</param>
    /// <returns><see cref="bool.TrueString"/>, если совпадают, и <see cref="bool.FalseString"/>, если не совпадают</returns>
    public bool IsEqual (FileHash otherHash) =>
        string.Equals(ToString(), otherHash.ToString(), StringComparison.OrdinalIgnoreCase);
}